from datetime import datetime
from django.db import models
from picklefield.fields import PickledObjectField
from omblog import listeners
from omblog import managers


class Tag(models.Model):
    slug = models.SlugField(
                db_index=True)
    tag = models.CharField(
                max_length=255)

    objects = managers.TagManager()

    def __unicode__(self):
        return u'%s' % self.tag

    @models.permalink
    def get_absolute_url(self):
        return ('omblog:tag', (), {
            'slug': self.slug,
        })


class Post(models.Model):
    IDEA = 1
    DRAFT = 2
    PUBLISHED = 3
    HIDDEN = 4
    STATUS_CHOICES = (
        (IDEA, 'Idea'),
        (DRAFT, 'Draft'),
        (PUBLISHED, 'Published'),
        (HIDDEN, 'Hidden'),
    )
    title = models.CharField(
                max_length=255)
    slug = models.SlugField(
                db_index=True,
                unique=True,
                max_length=255)
    description = models.CharField(
                max_length=255,
                blank=True,
                null=True)
    source_content = models.TextField('content')
    rendered_content = models.TextField(
                blank=True,
                null=True)
    status = models.PositiveSmallIntegerField(
                choices=STATUS_CHOICES,
                default=IDEA)
    tags = models.ManyToManyField(
                Tag,
                blank=True,
                null=True)
    created = models.DateTimeField(
                blank=True,
                null=True)

    objects = managers.PostManager()

    class Meta:
        ordering = ['-created']

    def __unicode__(self):
        return '%s' % self.title

    @models.permalink
    def get_absolute_url(self):
        return ('omblog:post', (), {
            'slug': self.slug,
        })

    def edit_url(self):
        return '/admin/omblog/post/%s/' % self.id

    def save(self, *args, **kwargs):
        if not self.created:
            self.created = datetime.now()
        super(Post, self).save(*args, **kwargs)



class PostVersion(models.Model):
    instance = PickledObjectField()
    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ['-created']

    def __unicode__(self):
        return u'%s : %s' % (self.instance.title, self.created.strftime('%c'))


models.signals.pre_save.connect(
                listeners.clear_cached_posts,
                sender=Post,
                dispatch_uid='clear_cached_posts')

models.signals.pre_save.connect(
                listeners.post_date,
                sender=Post,
                dispatch_uid='post_date')

models.signals.pre_save.connect(
                listeners.save_version,
                sender=Post,
                dispatch_uid='save_version')

models.signals.pre_save.connect(
                listeners.post_render_content,
                sender=Post,
                dispatch_uid='post_render_content')



from django.conf import settings as django_settings

"""rss title"""
RSS_TITLE = getattr(django_settings,
                            'OMBLOG_RSS_TITLE',
                            'RSS Feed')

"""rss link"""
RSS_LINK = getattr(django_settings,
                            'OMBLOG_RSS_LINK',
                            '/blog/')
"""cache timeout"""
CACHE_TIMEOUT = getattr(django_settings,
                            'OMBLOG_CACHE_TIMEOUT',
                           60)
"""Cache Enabled?"""
CACHE_ENABLED = getattr(django_settings,
                            'OMBLOG_CACHE_ENABLED',
                            True)
"""Cache prefix"""
CACHE_PREFIX = getattr(django_settings,
                            'OMBLOG_CACHE_PREFIX',
                            'omblog_')

"""Show the drafts if logged in"""
SHOW_IDEAS_IF_LOGGED_IN = getattr(django_settings,
                            'OMBLOG_SHOW_IDEAS_IF_LOGGED_IN',
                            True)

"""Show the drafts if logged in"""
SHOW_DRAFTS_IF_LOGGED_IN = getattr(django_settings,
                            'OMBLOG_SHOW_DRAFTS_IF_LOGGED_IN',
                            True)

"""Show the hidden if logged in"""
SHOW_HIDDEN_IF_LOGGED_IN = getattr(django_settings,
                            'OMBLOG_SHOW_HIDDEN_IF_LOGGED_IN',
                            True)


"""Total items to show on the index page"""
INDEX_ITEMS = getattr(django_settings,
                        'OMBLOG_INDEX_ITEMS',
                        20)



from django.contrib import admin
from omblog.models import Tag, Post
from omblog.forms import PostForm
from omblog.models import PostVersion


class TagAdmin(admin.ModelAdmin):
    prepopulated_fields = ({'slug' : ('tag',)})
    fieldsets = (
        (None, {
            'fields': ('tag',
                        'slug')
        }),
    )


class PostVersionAdmin(admin.ModelAdmin):
    pass


class PostAdmin(admin.ModelAdmin):
    date_hierarchy = 'created'
    form = PostForm
    prepopulated_fields = ({'slug' : ('title',)})
    list_display = ['title', 'description', 'created']
    search_fields = ['title', 'description']
    list_filter = ['status']
    filter_horizontal = ['tags']
    fieldsets = (
        (None, {
            'classes': ('content',),
            'fields': ('title',
                        'description',
                        'source_content',
                        'tags',
                        'status')
        }),
        ('Misc', {
            'classes': ('collapse',),
            'fields': ('slug',
                        'rendered_content',
                        'created')
        }),
    )


admin.site.register(Tag, TagAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(PostVersion, PostVersionAdmin)



from django.test import TestCase
from omblog.models import Post


class ModelTestCases(TestCase):

    def test_edit_url(self):
        """edit url returns correctly"""
        p = Post()
        p.title  = '1'
        p.slug = '1'
        p.content = '1'
        p.save()
        # edit url is correct
        self.assertEqual(
                p.edit_url(),
                '/admin/omblog/post/1/')





from haystack import indexes
from haystack import site
from omblog.models import Post

class PostIndex(indexes.SearchIndex):
    text = indexes.CharField(
            document=True,
            use_template=True)
    title = indexes.CharField(
            model_attr='title')
    description = indexes.CharField(
            model_attr='description')
    created = indexes.DateTimeField(
            model_attr='created')

    def index_queryset(self):
        return Post.objects.filter(status=Post.PUBLISHED)

site.register(Post, PostIndex)



from datetime import datetime
from django.test import TestCase
from django.db import models
from django.core.cache import cache
from django.conf import settings
from django.core.urlresolvers import reverse
from omblog.tests.mock import MockRequest
from omblog import listeners
from omblog import settings as o_settings
from omblog.models import Post
from omblog import views





class DecoratorCacheDisabledTestCases(TestCase):

    def setUp(self):
        # disable the cache
        settings.OMBLOG_CACHE_ENABLED = False
        reload(o_settings)
        # a mock factory
        self.rf = MockRequest()
        # disconnect the signals that clear the cache
        models.signals.pre_save.disconnect(
            listeners.clear_cached_posts,
            sender=Post,
            dispatch_uid='clear_cached_posts')


    def tearDown(self):
        del(settings.OMBLOG_CACHE_ENABLED)
        reload(o_settings)
        # re-attach the listeners
        models.signals.pre_save.connect(
                listeners.clear_cached_posts,
                sender=Post,
                dispatch_uid='clear_cached_posts')

    def test_cache_page_month_cache_enabled_false(self):
        """if no cache is enabled in the settings then
        no caching takes place"""
        # make the date
        d = datetime(
                year=2012,
                month=6,
                day=18,
                minute=2,
                hour=3,
                second=10)
        # request the page
        request = self.rf.get(
                reverse('omblog:month', args=[d.year,d.month]))
        response = views.month(request, d.year, d.month)
        self.assertNotContains(response, 'THISISATESTPOST')
        # now create a blog post
        p = Post()
        p.title = 'THISISATESTPOST'
        p.slug = 'slug'
        p.content = 'This is a test post'
        p.status = p.PUBLISHED
        p.description = 'test post'
        p.created = d
        p.save()
        # now it contains the content
        response = views.month(request, d.year, d.month)
        self.assertContains(response, 'THISISATESTPOST')
        # enabled it again
        settings.OMBLOG_CACHE_ENABLED = True
        reload(o_settings)
        # change the post to hidden and get view again
        p.status = p.HIDDEN
        p.save()
        # not contains
        response = views.month(request, d.year, d.month)
        self.assertNotContains(response, 'THISISATESTPOST')
        # change post status again and save
        p.status = p.PUBLISHED
        p.save()
        # not contains, because the page is cached
        response = views.month(request, d.year, d.month)
        self.assertNotContains(response, 'THISISATESTPOST')

    def test_cache_page_index_cache_enabled_false(self):
        """if no cache is enabled in the settings then
        no caching takes place"""
        # request the page
        request = self.rf.get(reverse('omblog:index'))
        response = views.index(request)
        self.assertNotContains(response, 'THISISATESTPOST')
        # now create a blog post
        p = Post()
        p.title = 'THISISATESTPOST'
        p.slug = 'slug'
        p.content = 'This is a test post'
        p.status = p.PUBLISHED
        p.description = 'test post'
        p.save()
        # now it contains the content
        response = views.index(request)
        self.assertContains(response, 'THISISATESTPOST')
        # enabled it again
        settings.OMBLOG_CACHE_ENABLED = True
        reload(o_settings)
        # change the post to hidden and get view again
        p.status = p.HIDDEN
        p.save()
        # not contains
        response = views.index(request)
        self.assertNotContains(response, 'THISISATESTPOST')
        # change post status again and save
        p.status = p.PUBLISHED
        p.save()
        # not contains, because the page is cached
        response = views.index(request)
        self.assertNotContains(response, 'THISISATESTPOST')



from django.contrib.syndication.views import Feed
from omblog.models import Post
from omblog import settings

class PostFeed(Feed):
    title = settings.RSS_TITLE
    link = settings.RSS_LINK 

    def items(self):
        return Post.objects.published()[:20]



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Tag'
        db.create_table('omblog_tag', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=50)),
            ('tag', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('omblog', ['Tag'])

        # Adding model 'Post'
        db.create_table('omblog_post', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=255)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('source_content', self.gf('django.db.models.fields.TextField')()),
            ('rendered_content', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('status', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=1)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
        ))
        db.send_create_signal('omblog', ['Post'])

        # Adding M2M table for field tags on 'Post'
        db.create_table('omblog_post_tags', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('post', models.ForeignKey(orm['omblog.post'], null=False)),
            ('tag', models.ForeignKey(orm['omblog.tag'], null=False))
        ))
        db.create_unique('omblog_post_tags', ['post_id', 'tag_id'])

    def backwards(self, orm):
        # Deleting model 'Tag'
        db.delete_table('omblog_tag')

        # Deleting model 'Post'
        db.delete_table('omblog_post')

        # Removing M2M table for field tags on 'Post'
        db.delete_table('omblog_post_tags')

    models = {
        'omblog.post': {
            'Meta': {'ordering': "['-created']", 'object_name': 'Post'},
            'created': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'rendered_content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            'source_content': ('django.db.models.fields.TextField', [], {}),
            'status': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '1'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['omblog.Tag']", 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'omblog.tag': {
            'Meta': {'object_name': 'Tag'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'tag': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['omblog']


import ccsitemaps
from omblog.models import Post


class PostSiteMap(ccsitemaps.SiteMap):
    model = Post

    @staticmethod
    def last_mod():
        try:
            last_mod = Post.objects\
                .published()\
                .order_by('-created')[0]
            return last_mod.created
        except IndexError:
            return None

    @staticmethod
    def get_objects():
        return Post.objects.published()

ccsitemaps.register(PostSiteMap)



from django.conf.urls import patterns, url
from omblog.feeds import PostFeed


urlpatterns = patterns('',
    url(r'^feed\.rss$',
        PostFeed(),
        name='feed'),
    url(r'^login/$',
        'django.contrib.auth.views.login',
        {'template_name': 'omblog/login.html'},
        name='login'),
    url(r'^edit/(?P<pk>\d+)/$',
        'omblog.views.edit',
        name='edit'),
    url(r'^tags/(?P<slug>[\w\-]+)/$',
        'omblog.views.tag',
        name='tag'),
    url(r'^(?P<slug>[\w\-]+)/$',
        'omblog.views.post',
        name='post'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{1,2})/$',
        'omblog.views.month',
        name='month'),
    url(r'^$',
        'omblog.views.index',
        name='index'),
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'PostVersion.post'
        db.add_column('omblog_postversion', 'post',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=1, to=orm['omblog.Post']),
                      keep_default=False)

    def backwards(self, orm):
        # Deleting field 'PostVersion.post'
        db.delete_column('omblog_postversion', 'post_id')

    models = {
        'omblog.post': {
            'Meta': {'ordering': "['-created']", 'object_name': 'Post'},
            'created': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'rendered_content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            'source_content': ('django.db.models.fields.TextField', [], {}),
            'status': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '1'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['omblog.Tag']", 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'omblog.postversion': {
            'Meta': {'ordering': "['-created']", 'object_name': 'PostVersion'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('picklefield.fields.PickledObjectField', [], {}),
            'post': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['omblog.Post']"})
        },
        'omblog.tag': {
            'Meta': {'object_name': 'Tag'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'tag': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['omblog']


from django import forms
from omblog.models import Post

class PostForm(forms.ModelForm):

    class Meta:
        model = Post

    class Media:
        css = {
            'all': ('omblog/css/omblog.writing.css',)
        }
        js = ('omblog/js/omblog.writing.js',)



from django.conf import settings as django_settings
from omblog import settings as omblog_settings



def get_key(key):
    try:
        django_prefix = django_settings.CACHES['default']['KEY_PREFIX']
    except KeyError:
        django_prefix = ''
    return '%s_%s_%s' % (django_prefix, omblog_settings.CACHE_PREFIX, key)



from django.core.cache import cache
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from omblog import settings as o_settings
from omblog.cache import get_key


def login_required(fn):
    """will redirect to the admin login page if a user 
    is not authed"""
    def _login_required(request, *args, **kwargs):
        if not request.user.is_authenticated():
            url = '%s?next=%s' % (
                reverse('omblog:login'),
                request.META.get('PATH_INFO')
            )
            return HttpResponseRedirect(url)
        return fn(request, *args, **kwargs)
    return _login_required

def cache_page(fn):
    """If a cached version of the page exists - return that
    unless the kill clearcache parameter is in the get and
    the user is auth'd.  """

    def _cache_page(request, *args, **kwargs):

        if o_settings.CACHE_ENABLED == False:
            return fn(request, *args, **kwargs)

        if request.user.is_authenticated():
            return fn(request, *args, **kwargs)

        signature = '%s_%s' % (fn.__name__, '_'.join(kwargs.values()))
        key = get_key(signature)
        if 'clearcache' not in request.GET.keys():
            cached = cache.get(key)
            if cached:
                return cached
        response = fn(request, *args, **kwargs)
        cache.set(key, response, o_settings.CACHE_TIMEOUT)
        return response

    return _cache_page



from django import template
register = template.Library()

class VideoNode(template.Node):
    
    def __init__(self, h264, webm, poster):
        self.h264 = h264
        self.webm = webm
        self.poster = poster

    def render(self, context):
        context = {
            'webm': self.webm,
            'h264': self.h264,
            'poster': self.poster
        }
        return """<video controls="controls" poster="%(poster)s" preload>
<source src="%(h264)s" type='video/mp4; codecs="avc1.42E01E,mp4a.40.2"'>
<source src="%(webm)s"  type='video/webm; codecs="vp8, vorbis"'>
<object id="flashvideo" width="720" height="540" data="http://releases.flowplayer.org/swf/flowplayer-3.2.10.swf" type="application/x-shockwave-flash">
<param name="movie" value="http://releases.flowplayer.org/swf/flowplayer-3.2.10.swf" >
<param name="allowfullscreen" value="true" />
<param name="allowscriptaccess" value="always" />
<param name="flashvars" value='config={"clip":{"url":"%(h264)s'>
</object>
</video>""" % context




@register.tag
def video(parser, token):
    bits = token.contents.split()
    return VideoNode(bits[1], bits[2], bits[3])



@register.simple_tag
def post_status(post):
    return post.get_status_display().lower()



# -*- coding: utf-8 -*-
from datetime import datetime
from django.test import TestCase
from omblog.models import Post


class CustomFilterTestCases(TestCase):
    
    def setUp(self):
        """setup stuff"""

    def tearDown(self):
        """teardown stuff"""
        
    
    def test_video_tag(self):
        """The video tag allows a html5 video tag to be outputted using
        {% video http://h264.url http://webm.url http://poster.url %}
        """
        p = Post()
        p.title = 'test post '
        p.source_content = '{% video http://w.video.mp4 http://w.video.webm http://w.poster.jpg %}'
        p.save()
        html = """
		<video controls="controls" poster="http://w.poster.jpg" preload>
			<source src="http://w.video.mp4" type='video/mp4; codecs="avc1.42E01E,mp4a.40.2"'>
			<source src="http://w.video.webm"  type='video/webm; codecs="vp8, vorbis"'>
			<object id="flashvideo" width="720" height="540" data="http://releases.flowplayer.org/swf/flowplayer-3.2.10.swf" type="application/x-shockwave-flash">
				<param name="movie" value="http://releases.flowplayer.org/swf/flowplayer-3.2.10.swf" >
				<param name="allowfullscreen" value="true" />
				<param name="allowscriptaccess" value="always" />
				<param name="flashvars" value='config={"clip":{"url":"http://w.video.mp4'>
			</object>
		</video>"""
        self.assertHTMLEqual(p.rendered_content, html)



import datetime
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from omblog import settings
from omblog.decorators import cache_page, login_required
from omblog.models import Post, Tag
from omblog import settings as om_settings

def login(request):
    """Authenticates a user and logs them in"""


@login_required
def edit(request, pk):
    # get the post
    get_object_or_404(Post, pk=pk)
    # 
    return render_to_response('omblog/edit.html', {
        'post': post,
    },
    context_instance=RequestContext(request))


@cache_page
def tag(request, slug):
    tag = get_object_or_404(Tag, slug=slug)
    posts = Post.objects.visible(user=request.user)
    posts = posts.filter(tags=tag)
    
    return render_to_response('omblog/tags.html', {
        'posts': posts,
        'dates': Post.objects.dates(),
        'tags': Tag.objects.tags_and_counts(),
        'tag': tag
    },
    context_instance=RequestContext(request))

@cache_page
def index(request):
    """The index"""
    posts = Post.objects.visible(user=request.user)
    if posts.count() > om_settings.INDEX_ITEMS:
        posts = posts[:om_settings.INDEX_ITEMS]
    return render_to_response('omblog/index.html', {
        'posts': posts,
        'dates': Post.objects.dates(),
        'tags': Tag.objects.tags_and_counts(),
    },
    context_instance=RequestContext(request))

@cache_page
def month(request, year, month):
    """The month archive"""
    date = datetime.date(year=int(year), month=int(month), day=1)
    posts = Post.objects.visible(user=request.user).filter(
                                    created__year=year,
                                    created__month=month)

    return render_to_response('omblog/month.html', {
        'posts': posts,
        'dates': Post.objects.dates(),
        'tags': Tag.objects.tags_and_counts(),
        'date': date
    },
    context_instance=RequestContext(request))


@cache_page
def post(request, slug):
    """view post"""
    try:
        post = Post.objects.visible(user=request.user).select_related('tags').get(slug=slug)
    except Post.DoesNotExist:
        raise Http404
    return render_to_response('omblog/post.html', {
        'post': post,
        'dates': Post.objects.dates(),
        'tags': Tag.objects.tags_and_counts(),
    },
    context_instance=RequestContext(request))




import os
from distutils.core import setup
from omblog import VERSION

root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)

setup(
    name='django-omblog',
    version=VERSION,
    license = 'BSD 3 Clause',
    description='A speedy django blog',
    url='https://github.com/obscuremetaphor/django-omblog',
    author='Obscure Metaphor',
    author_email='hello@obscuremetaphor.co.uk',
    package_data = {
        'omblog' : [
            'templates/omblog/*.html',
            'templates/search/indexes/omblog/*.txt',
            'static/omblog/css/*',
            'static/omblog/js/*',
            'static/omblog/img/*',
            'static/omblog/font/*',
        ]
    },
    packages=[
        'omblog',
        'omblog.templatetags',
        'omblog.tests',
        'omblog.migrations',
    ],
    install_requires = ['beautifulsoup4',
                        'django-picklefield',
                        'pygments',
                        'markdown'],
    classifiers = [
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django',]
        
)



import operator
from collections import OrderedDict
from django.core.cache import cache
from django.db import models
from omblog.cache import get_key
from omblog import settings as o_settings


class TagManager(models.Manager):

    def tags_and_counts(self):
        """returns tags with count of how many posts are tagged with it"""
        if o_settings.CACHE_ENABLED:
            key = get_key('tags_and_counts')
            cached = cache.get(key)
            if cached:
                return cached
        tags = []
        tag_models = super(TagManager, self).get_query_set().all()
        for tag in tag_models:
            tags.append((tag,tag.post_set.filter(status=3).count()))
        tags = sorted(tags, key=operator.itemgetter(1), reverse=True)

        if o_settings.CACHE_ENABLED:
            cache.set(key, tags, o_settings.CACHE_TIMEOUT)
        return tags


class PostManager(models.Manager):
    #
    # TODO: figure out how to decorate these 
    # @cached('archive_dates)
    #
    def dates(self):
        """returns the years and months for which there are posts"""
        if o_settings.CACHE_ENABLED:
            key = get_key('posts_dates')
            cached = cache.get(key)
            if cached:
                return cached

        posts = self.published()
        dates = OrderedDict()
        for post in posts:
            key = post.created.strftime('%Y_%m')
            try:
                dates[key][1] = dates[key][1] + 1
            except KeyError:
                dates[key] = [post.created, 1]

        if o_settings.CACHE_ENABLED:
            cache.set(key, dates, o_settings.CACHE_TIMEOUT)
        return dates


    def published(self):
        """returns all published blog post"""
        if o_settings.CACHE_ENABLED:
            key = get_key('posts_published')
            cached = cache.get(key)
            if cached:
                return cached

        posts = super(PostManager, self).get_query_set().filter(
                    status=self.model.PUBLISHED)

        if o_settings.CACHE_ENABLED:
            cache.set(key, posts, o_settings.CACHE_TIMEOUT)
        return posts


    def idea(self):
        """returns all idea blog posts"""
        if o_settings.CACHE_ENABLED:
            key = get_key('posts_idea')
            cached = cache.get(key)
            if cached:
                return cached

        posts = super(PostManager, self).get_query_set().filter(
                    status=self.model.IDEA)

        if o_settings.CACHE_ENABLED:
            cache.set(key, posts, o_settings.CACHE_TIMEOUT)
        return posts

    def draft(self):
        """returns all draft blog posts"""
        if o_settings.CACHE_ENABLED:
            key = get_key('posts_draft')
            cached = cache.get(key)
            if cached:
                return cached

        posts = super(PostManager, self).get_query_set().filter(
                    status=self.model.DRAFT)

        if o_settings.CACHE_ENABLED:
            cache.set(key, posts, o_settings.CACHE_TIMEOUT)
        return posts

    def hidden(self):
        """returns all hidden blog posts"""
        if o_settings.CACHE_ENABLED:
            key = get_key('posts_hidden')
            cached = cache.get(key)
            if cached:
                return cached

        posts = super(PostManager, self).get_query_set().filter(
                    status=self.model.HIDDEN)

        if o_settings.CACHE_ENABLED:
            cache.set(key, posts, o_settings.CACHE_TIMEOUT)
        return posts

    def visible(self, user):
        """returns all of the posts that are visible to the user"""
        posts = self.published()

        if o_settings.SHOW_DRAFTS_IF_LOGGED_IN \
                and user.is_authenticated():
            posts = posts | self.draft()

        if o_settings.SHOW_IDEAS_IF_LOGGED_IN \
                and user.is_authenticated():
            posts = posts | self.idea()

        if o_settings.SHOW_HIDDEN_IF_LOGGED_IN \
                and user.is_authenticated():
            posts = posts | self.hidden()

        return posts




# -*- coding: utf-8 -*-
from datetime import datetime
from django.test import TestCase
from omblog.models import Post, PostVersion



class ListenerTestCases(TestCase):

    def test_date_if_blank(self):
        """If no date is supplied for a post then datetime.now() is applied"""
        post = Post()
        post.title = 'Test'
        post.slug = 'test'
        post.source_content = "#Hello"
        post.save()
        # we now have a date
        self.assertTrue(post.created)
    
    def test_use_date_if_supplied(self):
        """If a date is supplied at the time of creation then it will be used
        when saving"""
        post_date = datetime.now()
        post = Post()
        post.title = 'Test'
        post.slug = 'test'
        post.source_content = "#Hello"
        post.created = post_date
        post.save()
        # we now have a date
        self.assertEqual(post_date, post.created)
    
    def test_version_saving(self):
        """The post model saves a version of itself when ever a post is saved """
        self.assertEqual(0, PostVersion.objects.count())
        post = Post()
        post.title = 'Test'
        post.slug = 'test'
        post.source_content = "#Hello"
        post.save()
        self.assertEqual(1, PostVersion.objects.count())

    def test_markdown_rendering(self):
        """The source is rendered into markdown on saving"""
        post = Post()
        post.title = 'Test'
        post.slug = 'test'
        post.source_content = "#Hello"
        post.save()
        # we now have markdown
        self.assertHTMLEqual('<h1>Hello</h1>', post.rendered_content)



import re
import pygments
import markdown
from bs4 import BeautifulSoup
from datetime import datetime
from pygments import lexers, formatters
from django import template
from django.core.urlresolvers import reverse
from django.core.cache import cache
from omblog.cache import get_key


def save_version(sender, instance, **kwargs):
    """Whena post is saved, so is a version of it
    """
    from omblog.models import PostVersion
    v = PostVersion()
    v.instance = instance
    v.save()

def clear_cached_posts(sender, instance, **kwargs):
    """when a model is saved clear the cache on the query and the cache on it's
    rendered page
    
    - archive_dates
    - posts_published
    - posts_idea
    - posts_draft
    - posts_hidden
    - tags_and_counts
    - view post
    - archive month
    - all tags for post
    """
    try:
        saved_instance = sender.objects.get(pk=instance.pk)
    except sender.DoesNotExist:
        saved_instance = None
    #
    # TODO: only kill the cache if items have changed .
    # e.g if the description, title, slug has changed nuke the 
    #       index
    #       archive
    #
    kill_list = []
    # dates
    if getattr(saved_instance, 'created', None) != instance.created:
        kill_list.append(get_key('archive_dates'))
        kill_list.append(get_key('month_%s_%s' % (
                        instance.created.year,
                        instance.created.month)))
    try:
        kill_list.append(get_key('month_%s_%s' % (
                        saved_instance.created.year,
                        saved_instance.created.month)))
    except AttributeError:
        pass
    # Just nuke all of the below for now
    kill_list.append(get_key('posts_published'))
    kill_list.append(get_key('posts_idea'))
    kill_list.append(get_key('posts_draft'))
    kill_list.append(get_key('posts_hidden'))
    kill_list.append(get_key('tags_and_counts'))
    kill_list.append(get_key('index_'))
    # always nuke the post
    kill_list.append(get_key('post_%s' % instance.slug ))
    # nuke tags
    try:
        for tag in instance.tags.all():
            kill_list.append(get_key('tag_%s' % tag))
    except ValueError:
        pass
    cache.delete_many(kill_list)


def post_date(sender, instance, **kwargs):
    """add a date to a post object if none is offered"""
    if instance.created is None:
        instance.created = datetime.now()


def post_render_content(sender, instance, **kwargs):
    """render the post content on save.
    Converts all code blocks to syntax highlighted pygments
    """
    content = instance.source_content
    code_regex = re.compile(r'(<code(.*?)</code>)', re.DOTALL)
    code_blocks = []
    for i, m in enumerate(code_regex.finditer(content)):
        code = m.group(0)
        # if there is a lang attribute, use that
        if code.find('lang') > 0:
            soup = BeautifulSoup(code)
            language = soup.code['lang'] 
            # TODO: catch errors here and provide a default lexer
            lexer = lexers.get_lexer_by_name(language)
            # remove the code tags
            code = code.replace('</code>', '')
            code = code.replace('<code>', '')
            code = re.sub('<code(.*?)>', '', code)
            # create the pygmented code with the lexer
            pygmented_string = pygments.highlight(
                    code,
                    lexer,
                    formatters.HtmlFormatter())
            # put the code blocks into the list for processintg later
            code_blocks.append(pygmented_string)
        else:
            # this is for the inline code snippets
            code_blocks.append(code)
        # replace the <code> tags with placeholders that can be used to replace
        content = content.replace(m.string[m.start():m.end()], 'OMCB|%s|OMCB' % i)
    # render the content now that code is out of the way
    # this is very hacky - must find a better way to do it
    c = template.Context({'instance': instance})
    content = '{% load omblog_tags %}' + content
    t = template.Template(content)
    content = t.render(c)
    # replaced placeholders with the actual code
    for i, code in enumerate(code_blocks):
        content = content.replace('<p>OMCB|%s|OMCB</p>' % i, code)
        content = content.replace('OMCB|%s|OMCB' % i, code)
    # do the mark down
    instance.rendered_content = markdown.markdown(content)



# -*- coding: utf-8 -*-
from django.test import TestCase
from omblog.models import Post

class ManagerTestCases(TestCase):
    
    def setUp(self):
        self.p1 = Post(
            title='test post',
            source_content='test',
            slug='test1')
        self.p1.save()
    
    def tearDown(self):
        self.p1.status = Post.DRAFT
        self.p1.save()

    #
    # TODO: test the visible manager method
    #

    def test_published(self):
        """published manager method returns only published blog posts"""
        # none
        self.assertEqual(0, Post.objects.published().count())
        # set to published
        self.p1.status = Post.PUBLISHED
        self.p1.save()
        # now one
        self.assertEqual(1, Post.objects.published().count())

    def test_idea(self):
        """published manager method returns only idea blog posts"""
        # one
        self.assertEqual(1, Post.objects.idea().count())
        # set to published
        self.p1.status = Post.PUBLISHED
        self.p1.save()
        # now none
        self.assertEqual(0, Post.objects.idea().count())

    def test_draft(self):
        """published manager method returns only draft blog posts"""
        # none
        self.assertEqual(0, Post.objects.draft().count())
        # set to published
        self.p1.status = Post.DRAFT
        self.p1.save()
        # now one
        self.assertEqual(1, Post.objects.draft().count())

    def test_hidden(self):
        """published manager method returns only hidden blog posts"""
        # none
        self.assertEqual(0, Post.objects.hidden().count())
        # set to published
        self.p1.status = Post.HIDDEN
        self.p1.save()
        # now one
        self.assertEqual(1, Post.objects.hidden().count())





# -*- coding: utf-8 -*-
from datetime import datetime
from django.test import TestCase
from django.conf import settings
from omblog import settings as omblog_settings

class SettingTestCases(TestCase):
    """Ensure that settings can be correctly overriden"""

    def test_override_cache_enabled(self):
        settings.OMBLOG_CACHE_ENABLED = False
        reload(omblog_settings)
        self.assertEqual(omblog_settings.CACHE_ENABLED, False)
        # revert it
        del(settings.OMBLOG_CACHE_ENABLED)

    def test_cache_enabled(self):
        reload(omblog_settings)
        self.assertEqual(omblog_settings.CACHE_ENABLED, True)

    def test_override_cache_prefix(self):
        settings.OMBLOG_CACHE_PREFIX = 'testerburger_'
        reload(omblog_settings)
        self.assertEqual(omblog_settings.CACHE_PREFIX, 'testerburger_')
        # revert it
        del(settings.OMBLOG_CACHE_PREFIX)

    def test_cache_prefix(self):
        reload(omblog_settings)
        self.assertEqual(omblog_settings.CACHE_PREFIX, 'omblog_')

    def test_override_show_ideas_if_logged_in(self):
        settings.OMBLOG_SHOW_IDEAS_IF_LOGGED_IN = False
        reload(omblog_settings)
        self.assertEqual(omblog_settings.SHOW_IDEAS_IF_LOGGED_IN, False)
        # revert it
        del(settings.OMBLOG_SHOW_IDEAS_IF_LOGGED_IN)

    def test_show_ideas_if_logged_in(self):
        reload(omblog_settings)
        self.assertEqual(omblog_settings.SHOW_IDEAS_IF_LOGGED_IN, True)

    def test_override_show_hidden_if_logged_in(self):
        settings.OMBLOG_SHOW_HIDDEN_IF_LOGGED_IN = False
        reload(omblog_settings)
        self.assertEqual(omblog_settings.SHOW_HIDDEN_IF_LOGGED_IN, False)
        # revert it
        del(settings.OMBLOG_SHOW_HIDDEN_IF_LOGGED_IN)

    def test_show_hidden_if_logged_in(self):
        reload(omblog_settings)
        self.assertEqual(omblog_settings.SHOW_HIDDEN_IF_LOGGED_IN, True)

    def test_override_show_drafts_if_logged_in(self):
        settings.OMBLOG_SHOW_DRAFTS_IF_LOGGED_IN = False
        reload(omblog_settings)
        self.assertEqual(omblog_settings.SHOW_DRAFTS_IF_LOGGED_IN, False)
        # revert it
        del(settings.OMBLOG_SHOW_DRAFTS_IF_LOGGED_IN)

    def test_show_drafts_if_logged_in(self):
        reload(omblog_settings)
        self.assertEqual(omblog_settings.SHOW_DRAFTS_IF_LOGGED_IN, True)

    def test_override_index_items(self):
        settings.OMBLOG_INDEX_ITEMS = 5
        reload(omblog_settings)
        self.assertEqual(omblog_settings.INDEX_ITEMS, 5)
        # revert it
        del(settings.OMBLOG_INDEX_ITEMS)

    def test_default_index_items(self):
        reload(omblog_settings)
        self.assertEqual(omblog_settings.INDEX_ITEMS, 20)
